/****************************************************************************
   Program:     $Id: util.c 617 2009-05-08 17:57:37Z rbeverly $
   Author:      Rob Beverly <rbeverly at csail.mit.edu>
   Date:        $Date: 2009-05-08 13:57:37 -0400 (Fri, 08 May 2009) $
   Description: Spoofer utility routines
****************************************************************************/
#include "spoof.h"
#include <stdarg.h>

/**
 * Fatal error condition function 
 */
void fatal(char *fmt, ...) {
    va_list ap;
    fflush(stdout);
    va_start(ap, fmt);
    fprintf(stderr, "*** ");
    vfprintf(stderr, fmt, ap);
    va_end(ap);
#ifdef _WIN32
    winpause("Press Enter to Exit.");
#endif
    exit(-1);
}

/**
 * Fatal error condition function; prints error
 */
void pfatal(char *fmt, ...) {
    va_list ap;
    fflush(stdout);
    va_start(ap, fmt);
    vfprintf(stderr, fmt, ap);
    fprintf(stderr, " ");
    perror("Error");
    va_end(ap);
    exit(-1);
}

/**
 * binDump -  Debug routine for examining raw packets/buffer/char strings 
 *
 * @param[in]   msg     pointer to message buffer
 * @param[in]   len     length of buffer to print
 */
void binDump(unsigned char *msg, int len) {
    int i;
    for (i=1;i<=len;i++) {
        if (len > 16) {
            printf("%02x ", *msg++);
            if (i%4 == 0) printf("\n");
        } else {
            printf("%02x", *msg++);
        }
    }
    printf("\n");
}

/**
 * byte32Swap - A ntohl() for big endian architectures who null macro it 
 */
unsigned long byte32Swap(unsigned long in) {
    return ( (in & 0x000000ff)<<24 | (in & 0x0000ff00)<<8 |
             (in & 0x00ff0000)>>8 | (in & 0xff000000)>>24 );
}

/**
 * byte16Swap - A ntoh() for big endian architectures who null macro it 
 */
unsigned short byte16Swap(unsigned short in) {
    return ( (in & 0x00ff)<<8 | (in & 0xff00)>>8 );
}

/**
 * checkroot -  Check for root priviledges 
 */
void checkroot() {
#ifndef _WIN32
    if ( (getuid() != 0) && (geteuid() != 0) ) {
        warn("This application needs root privileges to send probes");
        warn("with spooofed source addresses (using raw sockets).");
        warn("See: http://%s/faq.php for details.", REPORT_HOST);
        fatal("Must be run as root!\n");
    }
#endif
}

/**
 * win_init -  Windows socket API startup 
 */
void win_init() {
#ifdef _WIN32
    WSADATA wsd;
    if (WSAStartup(MAKEWORD(2, 2), &wsd) != 0) {
        winfatal("** WSAStartup() failed");
    }
#endif
}

/**
 * banner - Hi there 
 */
void banner() {
    printf(">> MIT ANA Spoofer Project\n");
    printf(">> IP Source Spoofing Tester v%s\n", PACKAGE_VERSION);
    printf(">> http://%s\n", REPORT_HOST);
    printf(">> Copyright (c) 2004-2009 Robert Beverly\n");
    printf(">>\n");
}

void banner2() {
    printf(">> This tester probes your ISP:\n");
    printf(">>\t- Sends non-spoofed source validation UDP packets\n");
    printf(">>\t- Sends spoofed source probes to multiple destinations\n");
    printf(">>\t- Runs traceroute to each destination\n");
    printf(">>\t- Returns a web page with a graphical picture of results\n");
    printf(">>\n");
}

/**
 * confirm - Ask for user input before proceeding
 */
void confirm() {
    char buf[SMALLBUF];

    printf("Continue? [y/N]....");
    fflush(stdout);
    fgets(buf, SMALLBUF, stdin);
    if ( ((buf[0] != 'y') && (buf[0] != 'Y')) || (strlen(buf) != 2) ) 
        fatal("Aborting.\n");
}

/**
 * spoofSleep -  Portable microsecond random sleep 
 */
void spoofSleep() {
    unsigned int gap;

    /* random interpacket delay 0-0.5s */
    gap = (unsigned int)((double)rand() / ((double)RAND_MAX + 1) * 500);
#ifdef _WIN32
    Sleep(gap);
#else
    usleep(gap * 1000);
#endif
}

/**
 * timestamp - form time/date string
 */
char *timestamp() {
   static char str[BUFSIZE];
   time_t clock;
   struct tm *t;

   clock = time(NULL);
   t = localtime(&clock);
   snprintf(str, sizeof(str), "%02d/%02d %02d:%02d:%02d", t->tm_mon + 1,
      t->tm_mday, t->tm_hour, t->tm_min, t->tm_sec);
   return (str);
}

/**
 * streamSetup - set stdout to be unbuffered, replicate stdout
 *               to the stderr file descriptor.  useful with
 *               the authorization services API (for osx gui)
 *               which doesn't pipe back stderr and prevents
 *               weird interleaving of stderr, stdout messages.
 */
void streamSetup() {
    if (setvbuf(stdout, NULL, _IONBF, 0) == EOF) {
        perror("setvbuf");
    }
    if (dup2(1,2) < 0) {
        perror("dup2");
    }
}

/* Unified IPv4/IPv6 address field helper routines.
 *   Ex:
 *     char bob[IPV6ADDRLEN];
 *     uint32_t alice = 91232354;
 *     h4to6n(alice, bob);
 *     printf("Alice: %u %u\n", alice, n6toh4(bob));
 */
/* network byte-order IPv6 to network byte-order IPv4 */
uint32_t n6ton4(unsigned char *addr) {
    unsigned char *begin = addr + (IPV6ADDRLEN - IPV4ADDRLEN);
    uint32_t res;
    memcpy(&res, begin, IPV4ADDRLEN);
    return (res);
}

/* network byte-order IPv6 to host byte-order IPv4 */
uint32_t n6toh4(unsigned char *addr) {
    return (ntohl(n6ton4(addr)));
}

/* network byte-order IPv4 to network byte-order IPv6 */
int n4ton6(uint32_t ip, unsigned char *addr) {
    unsigned char *begin = addr + (IPV6ADDRLEN - IPV4ADDRLEN);
    memcpy(begin, &ip, IPV4ADDRLEN);
    return TRUE;
}

/* host byte-order IPv4 to network byte-order IPv6 */
int h4ton6(uint32_t ip, unsigned char *addr) {
    unsigned char *begin = addr + (IPV6ADDRLEN - IPV4ADDRLEN);
    uint32_t res = htonl(ip);
    memcpy(begin, &res, IPV4ADDRLEN);
    return TRUE;
}

int n6tohexstring(unsigned char *addr, unsigned char *res) {
    int i=0;
    snprintf((char *)res, 3, "0x");
    for (i=1;i<=IPV6ADDRLEN;i++) {
        snprintf((char *)res + (i*2), 3, "%02x", *addr);
        addr++;
    }
    return TRUE;
}

int isIPv4(unsigned char *addr) {
    int i=0;
    unsigned char *d = addr;
    for (i=0;i<(IPV6ADDRLEN - IPV4ADDRLEN);i++) {
        if (*d != 0x00)
            return FALSE;
    }
    return TRUE;
}

int isIPv6(unsigned char *addr) {
    return (isIPv4(addr) ? FALSE : TRUE);
}

int isIPv6Equal(unsigned char *a1, unsigned char *a2) {
    int ret = memcmp(a1, a2, IPV6ADDRLEN);
    return (ret == 0 ? TRUE : FALSE);
}

char *printIPv6(unsigned char *a) {
    static char v6string[SMALLBUF];
    inet_ntop(AF_INET6, a, v6string, SMALLBUF);
    return v6string;
}
